@@ -39,6 +39,13 @@ def mk_notebook_context_menu(callback_object):
     mi.connect("activate", callback_object.on_new_tab)
     menu.add(mi)
     menu.add(Gtk.SeparatorMenuItem())
+    mi = Gtk.MenuItem(_("Save Tabs"))
+    mi.connect("activate", callback_object.on_save_tabs)
+    menu.add(mi)
+    mi = Gtk.MenuItem(_("Restore Tabs"))
+    mi.connect("activate", callback_object.on_restore_tabs_with_dialog)
+    menu.add(mi)
+    menu.add(Gtk.SeparatorMenuItem())
     mi = Gtk.ImageMenuItem("gtk-preferences")
     mi.set_use_stock(True)
     mi.connect("activate", callback_object.on_show_preferences)
