@@ -21,15 +21,19 @@ import code
 import logging
 import os
 import re
+import shlex
 import signal
 import subprocess
 import sys
 import threading
 import uuid
 
+from enum import IntEnum
 from pathlib import Path
 from typing import Optional
 from typing import Tuple
+from urllib.parse import unquote
+from urllib.parse import urlparse
 
 from time import sleep
 
@@ -85,6 +89,11 @@ __all__ = ['GuakeTerminal']
 # pylint: enable=anomalous-backslash-in-string
 
 
+class DropTargets(IntEnum):
+    URIS = 0
+    TEXT = 1
+
+
 class GuakeTerminal(Vte.Terminal):
 
     """Just a vte.Terminal with some properties already set.
@@ -95,8 +104,9 @@ class GuakeTerminal(Vte.Terminal):
         self.guake = guake
         self.configure_terminal()
         self.add_matches()
-        self.connect('button-press-event', self.button_press)
-        self.connect('child-exited', self.on_child_exited)
+        self.handler_ids = []
+        self.handler_ids.append(self.connect('button-press-event', self.button_press))
+        self.connect('child-exited', self.on_child_exited)  # Call on_child_exited, don't remove it
         self.matched_value = ''
         self.font_scale_index = 0
         self._pid = None
@@ -105,6 +115,16 @@ class GuakeTerminal(Vte.Terminal):
         self.found_link = None
         self.uuid = uuid.uuid4()
 
+        self.setup_drag_and_drop()
+
+    def setup_drag_and_drop(self):
+        self.targets = Gtk.TargetList()
+        self.targets.add_uri_targets(DropTargets.URIS)
+        self.targets.add_text_targets(DropTargets.TEXT)
+        self.drag_dest_set(Gtk.DestDefaults.ALL, [], Gdk.DragAction.COPY)
+        self.drag_dest_set_target_list(self.targets)
+        self.connect('drag-data-received', self.on_drag_data_received)
+
     def get_uuid(self):
         return self.uuid
 
@@ -331,6 +351,17 @@ class GuakeTerminal(Vte.Terminal):
             if self.get_pty() is not None:
                 libutempter.utempter_remove_record(self.get_pty().get_fd())
 
+    def on_drag_data_received(self, widget, drag_context, x, y, data, info, time):
+        if info == DropTargets.URIS:
+            uris = data.get_uris()
+            for uri in uris:
+                path = Path(unquote(urlparse(uri).path))
+                self.feed_child(shlex.quote(str(path.absolute())) + ' ')
+        elif info == DropTargets.TEXT:
+            text = data.get_text()
+            if text:
+                self.feed_child(text)
+
     def quick_open(self):
         self.copy_clipboard()
         clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
@@ -461,38 +492,29 @@ class GuakeTerminal(Vte.Terminal):
         # start_new_thread(self.delete_shell, (pid,))
 
     def delete_shell(self, pid):
-        """This function will kill the shell on a tab, trying to send
-        a sigterm and if it doesn't work, a sigkill. Between these two
-        signals, we have a timeout of 3 seconds, so is recommended to
-        call this in another thread. This doesn't change any thing in
-        UI, so you can use python's start_new_thread.
+        """Kill the shell with SIGHUP
+
+        NOTE: Leave it alone, DO NOT USE os.waitpid
+
+        > sys:1: Warning: GChildWatchSource: Exit status of a child process was requested but
+                 ECHILD was received by waitpid(). See the documentation of
+                 g_child_watch_source_new() for possible causes.
+
+        g_child_watch_source_new() documentation:
+            https://developer.gnome.org/glib/stable/glib-The-Main-Event-Loop.html#g-child-watch-source-new
+
+        On POSIX platforms, the following restrictions apply to this API due to limitations
+        in POSIX process interfaces:
+            ...
+            * the application must not wait for pid to exit by any other mechanism,
+              including waitpid(pid, ...) or a second child-watch source for the same pid
+            ...
+        For this reason, we should not call os.waitpid(pid, ...), leave it to OS
         """
         try:
             os.kill(pid, signal.SIGHUP)
         except OSError:
             pass
-        num_tries = 30
-
-        while num_tries > 0:
-            try:
-                # Try to wait for the pid to be closed. If it does not
-                # exist anymore, an OSError is raised and we can
-                # safely ignore it.
-                if os.waitpid(pid, os.WNOHANG)[0] != 0:
-                    break
-            except OSError:
-                break
-            sleep(0.1)
-            num_tries -= 1
-
-        if num_tries == 0:
-            try:
-                os.kill(pid, signal.SIGKILL)
-                os.waitpid(pid, 0)
-            except OSError:
-                # if this part of code was reached, means that SIGTERM
-                # did the work and SIGKILL wasnt needed.
-                pass
 
     def spawn_sync_pid(self, directory):
 
